//========================================================
// L1 solver..
//
// Solve the (over determained) linear system Ax = b by minimized the error 
// using The L1 metric. 
// M equations of K variables M > K
//
// The solvex uses single phase simplex algorithm to solve
// the problem:
//
// min: sum w|z|
// s.t.
// Ax + |z| = b
//
// where w is optional weight vector.
//
// This linear problem has the following proparties:
// 1. The size of the problem is MxN where N = 2(K + M).
// 2. It has an immediate basic feasible solution given by:
//    x = 0, |z| = b. 2K + M are have zero value M has non-zero value
//    which forms a basis. 
// 3. The problem has lower bound of 0 since the coeficients (w) of the objective
//    variables (z) are all positive.
// 4. (2) and (3) implies that a solution always exists and that one phase is sufficient.
//
// Double precission accuracy is used;
//
// (c) The Hebrew University of Jerusalem
// Moshe Ben-Ezra
//========================================================

 

#ifndef LPS_H
#define LPS_H

#define RET_OK   0
#define RET_CONT 1
#define RET_DIVZ 2
#define RET_FAIL 3

class solvex {

private:
  int lpwidth;			// tablue width  = N + 1
  int lpheight;			// tablue height = M + 1
  int lpvar_no;			// tabeau var numbers
  int var_no;			// orginal var number
  int equ_no;			// number of equations
  int cur_line;			// current equation
  int *B;			// index of the basic feasible solution.

  double *tab;			// tablue
  double *sol;			// solution

public:
  solvex(int m, int k);		// construct linear equation set of m equantions k variables
  ~solvex();			// distructor
  double *get_row(int i);	// row index


  int add_equ(double *p, double val, double w=1.0);	// add equation <Ai,Xi> =  bi
  void negate_row(double *p);						
  void multiply_row(double *p, double scalar);						
  void subtruct_row(double *src, double *dst, double scalar);

  int solve();			// solve lp program
  double get_obj();		// get objective function value (error function)
  double get_var(int i);	// get variable i value, first K are the solution
  void reset();			// reset the system for another set.
				// the rest M are the error values.
  void print(FILE *fp);
  int locate_pivot(int &col, int &row, double &pval);
  void pivot(int col, int row, double pval);
  };

#endif





